#include <fstream>
#include <iostream>
#include <string.h>
using namespace std;

const int CODL = 8;
const int MAXS = 100;
const int MAXO = 50;
const int MAXV = 80;

struct Vino {
  char nome[MAXS];
  int anno;
  float quantita;
  float prezzo;
  char codice[CODL];
};

struct Magazzino {
  Vino vi[MAXV];
  int numVi;
};

struct Ordine {
  char codice[CODL];
  float quantita;
};

struct CodaOrd {
  Ordine ord[MAXO];
  int front;
  int back;
};

struct Cantina {
  Magazzino mag;
  CodaOrd coda;
};

bool vocale(char c) {
  switch(c){
    case 'a': case 'A':
    case 'e': case 'E':
    case 'i': case 'I':
    case 'o': case 'O':
    case 'u': case 'U':
      return true;
  }
  return false;
}

bool consonante(char c) {
  return ((c>='a' && c<='z') || (c>='A' && c<='Z')) && !vocale(c);
}

void calcolaCodice(const char n[], int a, char cod[]) {
  int l = strlen(n);
  int c = 0;
  for(int i=0; i<l && c<5; i++)
    if(consonante(n[i]))
      cod[c++] = n[i];
  for(;c<5; c++)
    cod[c] = 'x';

  cod[c++] = ((a/10) % 10) + '0';
  cod[c++] = (a % 10) + '0';
  cod[c] = '\0';
}

int caricaVino(Cantina* pc,  const char nf[]) {

  fstream ff;
  ff.open(nf, ios::in);
  if(!ff) return 0;
  Vino v;
  int conta = 0;

  while(ff >> v.nome >> v.anno >> v.quantita >> v.prezzo && pc->mag.numVi < MAXV) {
    calcolaCodice(v.nome, v.anno, v.codice); 
    pc->mag.vi[pc->mag.numVi] = v;
    pc->mag.numVi++;
    conta++;
  }  
  return conta;
}

bool insert(CodaOrd* p, Ordine o) {
  if((p->back+1)%MAXO == p->front) 
    return false;
  p->ord[p->back] = o;
  p->back = (p->back+1) % MAXO;
  return true; 
}

int trovaVino(const Magazzino* pm, const char cod[]) {
  for(int i=0; i<pm->numVi; i++)
    if(strcmp(cod, pm->vi[i].codice) == 0)
      return i;
  return -1;
}

bool nuovoOrdine(Cantina* pc, Ordine o) {
  int i;
  i = trovaVino(&pc->mag, o.codice);
  if(i == -1) 
    return false;
  return insert(&pc->coda, o);
}


float evadi(Cantina* pc) {
  float tot = 0;
  while(pc->coda.front != pc->coda.back) {
    int t = trovaVino(&pc->mag, pc->coda.ord[pc->coda.front].codice);
    if(t == -1) return tot;
    if(pc->mag.vi[t].quantita < pc->coda.ord[pc->coda.front].quantita)
      return tot;
    pc->mag.vi[t].quantita -= pc->coda.ord[pc->coda.front].quantita;
    tot += pc->coda.ord[pc->coda.front].quantita * pc->mag.vi[t].prezzo;
    pc->coda.front = (pc->coda.front + 1) % MAXO;
  }
  return tot;
}

void ordina(Magazzino* pm) {
  for(int i=0; i<pm->numVi-1; i++) {
    int index = i;
    for(int j=i+1; j<pm->numVi; j++) 
      if(pm->vi[j].anno < pm->vi[index].anno)
        index = j;
    Vino tmp = pm->vi[i];
    pm->vi[i] = pm->vi[index];
    pm->vi[index] = tmp;
  }
}

void stampa(const Magazzino* pm) {
  for(int i=0; i<pm->numVi; i++){
    cout  << "Nome: " << pm->vi[i].nome << endl
          << "Anno: " << pm->vi[i].anno << endl
          << "Quantita: " << pm->vi[i].quantita << endl
          << "Prezzo: " << pm->vi[i].prezzo << endl
          << "Codice: " << pm->vi[i].codice << endl << endl;
  }
}

void stampa(const CodaOrd* pco) {
  for(int i=pco->front; i!=pco->back; i=(i+1)%MAXO)
    cout << "Codice: " << pco->ord[i].codice << endl
         << "Quantita: " << pco->ord[i].quantita << endl << endl;
}
  
void stampa(const Cantina* pc) {
  Magazzino aux;
  aux = pc->mag;
  ordina(&aux);
  cout << "\n##### Magazzino #####" << endl;
  stampa(&aux);
  cout << "\n##### Coda #####" << endl;
  stampa(&pc->coda);
}
